#!/usr/bin/env bash

# dsinfo.sh
#
# Collect Docker/System information

if [ "$(id -u)" != "0" ]; then
  printf "This script must be run as root or sudo!\n"
  exit 1
fi

while getopts ":t:o:l:bd" opt; do
  case $opt in
    b)
      BATCH=true
      ;;
	d)
	  DOCKER=true # When ran with -d, the script will also perform Docker API calls
	  ;;
    t)
      TD="$OPTARG"
      ;;
    o)
      TARFILE="$OPTARG"
      ;;
    l)
      LOGLINES="$OPTARG"
      ;;
    \?)
      echo "Invalid option: -$OPTARG" >&2
      exit 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument" >&2
      exit 1;
      ;;
  esac
done

BATCH=${BATCH:-false}
DOCKER=${DOCKER:-false}

# temp directory
TD="${TD:-/dsinfo}"
TR="$TD/dsinfo.txt"
TARFILE="${TARFILE:-/dsinfo.tar.gz}"

# number of log lines to collect
LOGLINES="${LOGLINES:-5000}"

# OS
OSNAME=$(cat /etc/*release | grep "^NAME" | cut -d'"' -f2)

bline() {
  echo "" >> ${TR}
}


header() {
  printf "%0.s=" $(seq 1 25) >> ${TR}
  echo "" >> ${TR}
  echo $* >> ${TR}
  printf "%0.s=" $(seq 1 25) >> ${TR}
  bline
}


execute() {
  bline
  command=$*
  echo ${command} >> ${TR}
  printf "%0.s~" $(seq 1 ${#command}) >> ${TR}
  bline

  ex=$(echo "${command}" | cut -d" " -f1)
  if command -v "${ex}" >/dev/null 2>&1; then
    eval "${command}" >> ${TR}
  else
    echo "'${ex}' command not found" >> ${TR}
  fi
}


set_opts() {
  dopts=$(cat $* | grep "^OPTIONS")

  if [ -z "${dopts}" ]; then
    echo "OPTIONS='-D'" >> $*
  else
    sed -i "s/^OPTIONS='/OPTIONS='-D /g" $*
  fi
}


set_dopts() {
  dopts=$(cat $* | grep "^DOCKER_OPTS")

  if [ -z "${dopts}" ]; then
    echo 'DOCKER_OPTS="-D"' >> $*
  else
    sed -i 's/^DOCKER_OPTS="/DOCKER_OPTS="-D /g' $*
  fi
}


if [ $BATCH != true ]; then
  # begin
  echo
  echo "*************************** dsinfo.sh ***************************"
  echo
  echo "Beginning data collection ..."
  echo

  # check debug
  # Note: Boot2Docker has debug enabled by default.
  dbug=$(docker info | grep "Debug mode (server)" | grep -o "true")

  if [ "${dbug}" != "true" ]; then
    echo
    echo "The Docker daemon is not running with debug logging enabled!"
    echo ""
    read -p "Enable debug logging and restart the Docker daemon? [y/n]: " answer
    case ${answer} in

      [yY] | [yY][Ee][Ss] )
        case ${OSNAME} in
          "CentOS Linux")
            set_opts '/etc/sysconfig/docker'
            systemctl restart docker
            ;;
          "Debian GNU/Linux")
            set_dopts '/etc/default/docker'
            service docker restart
            ;;
          "NAME=Fedora")
            set_opts '/etc/sysconfig/docker'
            systemctl restart docker
            ;;
          "Red Hat Enterprise Linux Server")
            set_opts '/etc/sysconfig/docker'
            systemctl restart docker
            ;;
          "Ubuntu")
            set_dopts '/etc/default/docker'
            service docker restart
            ;;
          *)
            echo
            echo "Can't determine the location of the Docker config file."
            echo "Enable debug logging manually and restart the Docker daemon"
            exit 1
            ;;
        esac

        echo
        echo "Debug logging enabled and daemon restarted."
        echo "Run the 'dsinfo.sh' data collection script again when you're ready ..."
        exit 1
        ;;
      *)
        echo
        echo "Continuing without enabling debug logging ..."
        ;;
    esac
  fi
fi
mkdir -p ${TD}
rm -r ${TD}/*

# tmp directories

if [ $DOCKER = true ] ; then
  mkdir -p ${TD}/inspect
  mkdir -p ${TD}/logs
fi

# Trigger a stack dump from any UCP controller
docker kill -s USR1 ucp-controller > /dev/null


# initialize
header 'Docker/System Information'
bline
echo $(date) >> ${TR}
bline

# docker
# ~~~~~~
if [ $DOCKER = true ]; then
	header 'Docker Info'

	execute 'docker version'
	execute 'docker info'
	execute 'docker images --no-trunc'
	execute 'docker stats --all --no-stream'
    if [ -f /etc/docker/daemon.json ]; then
        execute 'cat /etc/docker/daemon.json'
    fi
fi

execute '/check-config.sh'

# daemon logs
if [ -f /var/log/docker.log ] ; then
    tail -n ${LOGLINES} /var/log/docker.log > ${TD}/var_log_daemon.log
fi
journalctl -n ${LOGLINES} -u docker.service > ${TD}/journalctl_daemon.log
# Others?

# dmesg
dmesg | tail -n ${LOGLINES} > ${TD}/dmesg.txt

# daemon stack trace
# ~~~~~~~~~~~~~~~~~~
DAEMON_STACK_TRACES=()
update_daemon_stack_traces() {
  DAEMON_STACK_TRACES=()
  while read -r f ; do
    DAEMON_STACK_TRACES[${#DAEMON_STACK_TRACES[@]}]=$f
  done < <(find /var/run/docker -maxdepth 1 -type f -name 'goroutine-stacks-*log' | sort -r)
}

if [ -f /var/run/docker.pid ] ; then
  update_daemon_stack_traces
  OLD_DAEMON_STACK_TRACES=(${DAEMON_STACK_TRACES[*]})
  kill -s USR1 $(cat /var/run/docker.pid)
  TRY_COUNT=0
  # wait for the new stack trace to be present (up to 10s)
  while [[ ${#OLD_DAEMON_STACK_TRACES[@]} == ${#DAEMON_STACK_TRACES[@]} && ${TRY_COUNT} < 10 ]] ; do
    sleep 1s
    ((TRY_COUNT++))
    update_daemon_stack_traces
  done
  if [ ${#DAEMON_STACK_TRACES[@]} > 0 ] ; then
    mv ${DAEMON_STACK_TRACES[0]} ${TD}/daemon-stack-trace.log
  fi
fi

if [ $DOCKER = true ]; then
	UCP_CONTAINERS=$(docker ps -a --filter name=ucp- --format {{.Names}})
	DTR_CONTAINERS=$(docker ps -a --filter name=dtr- --format {{.Names}})
	for container in $UCP_CONTAINERS $DTR_CONTAINERS; do
          # For the ucp-controller, grab a current stack dump
          if [ "${container}" = "ucp-controller" ] ; then
              docker cp $container:/tmp/diag ${TD}/diag
          fi
          if [ "${container}" = "ucp-auth-store" ] ; then
              NODE_ADDR=$(docker info --format '{{.Swarm.NodeAddr}}')
              IMAGE_NAME=$(docker inspect ucp-auth-api --format '{{.Config.Image}}')
              docker run --rm -v ucp-auth-store-certs:/tls ${IMAGE_NAME} --db-addr=${NODE_ADDR}:12383 db-status > ${TD}/rethinkdb-status.json
          fi
    echo ${container} >> ${TD}/logs/${container}.log
	  docker logs --timestamps --tail ${LOGLINES} ${container} >> ${TD}/logs/${container}.log 2>&1
	  docker inspect ${container} >> ${TD}/inspect/${container}.txt 2>&1
          if echo $container | grep -q "ucp-hrm.*"; then
              docker cp $container:/usr/local/etc/haproxy/haproxy.cfg ${TD}/${container}-haproxy.cfg
          fi
	done
fi

# DTR 1.4.3
# ~~~
DTR=/usr/local/etc/dtr
if [ -d ${DTR} ]; then
	mkdir -p ${TD}/dtr/generatedConfigs
	mkdir -p ${TD}/dtr/logs
	cp ${DTR}/*.yml ${TD}/dtr
	cp ${DTR}/generatedConfigs/*.conf ${TD}/dtr/generatedConfigs
	cp ${DTR}/generatedConfigs/*.yml ${TD}/dtr/generatedConfigs

	for log in ${DTR}/logs/*
	do
		logbase=$(basename ${log})
		tail -n ${LOGLINES} ${log} > ${TD}/dtr/logs/${logbase}.log
	done
else
	bline
	header 'Docker Trusted Registry 1.4.3'
	bline
	echo "The '/usr/local/etc/dtr' directory does not exist." >> ${TR}
fi

# DTR 2.0.0+
# ~~~
header 'Docker Trusted Registry 2.0.0 or later'
docker network inspect dtr-br >> ${TR} 2>&1
docker network inspect dtr-ol >> ${TR} 2>&1

# system
# ~~~~~~
bline
bline
header 'System Info'
bline

execute 'hostname'
execute 'cat /etc/*release'
execute 'cat /proc/version'
execute 'cat /proc/cpuinfo'
execute 'cat /proc/meminfo'
execute 'cat /proc/cgroups'
execute 'cat /proc/self/cgroup'
execute 'df -h'
execute 'mount'
if [ $BATCH = false ]; then
  execute 'ifconfig'
  execute 'ps aux | grep docker'
  execute 'netstat -npl'
  execute 'sestatus'
fi
execute 'vmstat 1 5'
execute 'iostat 1 5'
execute 'dmidecode'

bline
header 'Network Information'
bline
/network-support.sh 1>> ${TR} 2>&1

if [ -n "$LOCAL_MANAGER_IP_ADDRESS" ]; then
  METRICS="$TD/metrics.txt"
  /bin/metrics.sh >> $METRICS 2>&1
fi

CERTDUMP="$TD/certdump.txt"
certdump --scan >> $CERTDUMP

# tar
BASEDIR=`basename $TD`
cd "${TD}/.." && tar -zcf $TARFILE $BASEDIR

if [ $BATCH = false ]; then
  # instructions
  echo <<EOF
Data collection complete ...

Notes
=====

This script created the directories:
  - /dsinfo
  - /dsinfo/dtr
  - /dsinfo/dtr/generatedConfigs
  - /dsinfo/dtr/logs
  - /dsinfo/inspect
  - /dsinfo/logs

This script collected the following:
  - Docker daemon configuration and logs
  - Inspect results and logs from the last 20 containers
  - Miscellaneous system information (Output to: /dsinfo/report.md)

All files/directories were compressed to: /dsinfo.tar.gz

---------------------------------------------------------------------------------

*** Important ***

Before sharing the dsinfo.tar.gz archive, review all collected files for
private information and edit/delete if necessary.

If you do edit/remove any files, recreate the tar file with the following command:

  sudo tar -zcf /dsinfo.tar.gz /dsinfo
EOF
else
  cat $TARFILE
fi
