#!/bin/sh

echo 'Please check the "ucp-controller" container logs for configuration logs for the HTTP Routing Mesh.'
# While the container is running, the ucp-controller container will add new
# config files by copying them into /etc/hrmconfig/haproxy.cfg. When this
# happens, we'll verify the config to make sure it has the right checksum
# and that it's a valid config file; if it is, we copy it to the final config
# location and reload haproxy with the new config.
CONFIG_LOCATION=/etc/hrmconfig/haproxy.cfg
CONFIG_CHECKSUM_LOCATION=/etc/hrmconfig/haproxy.sha
FINAL_CONFIG_LOCATION=/usr/local/etc/haproxy/haproxy.cfg
# The FINAL_CONFIG_LOCATION is what we use to run haproxy, but we also keep
# around a temporary version before we run setup_logging_in_config on it so
# we can check the checksum without the logging stuff we add.
TMP_CONFIG_LOCATION=/usr/local/etc/haproxy/haproxy.cfg.tmp
PIDFILE=/var/run/haproxy.pid
CHECKSUM=
if [[ -s $TMP_CONFIG_LOCATION ]]; then
    CHECKSUM=$(md5sum "$TMP_CONFIG_LOCATION" | awk '{print $1}')
    start_haproxy
fi

# logread can take a while before it can connect to the syslogd buffer so we
# retry until it succeeds, at which point it'll read from the logs forever.
read_logread() {
    while true; do
        logread -F
    done
}

if [[ $DEBUG ]]; then
    /sbin/syslogd -l 7 -C
    read_logread &
fi

# When DEBUG=1 is set in this task, we add in some logging configuration
# to the config files to turn on debug logging. If DEBUG is not set, we remove
# the "option tcplog" and "option httplog" settings in the HRM config because
# otherwise haproxy will complain.
setup_logging_in_config() {
    config=$1
    outfile=$2
    if [[ $DEBUG ]]; then
        echo "global" > $outfile
        echo "	log /dev/log local0" >> $outfile
        cat $1 >> $outfile
    else
        cat $1 | grep -v "option tcplog" | grep -v "option httplog" > $outfile
    fi
}

start_haproxy() {
    echo "Starting haproxy"
    setup_logging_in_config $TMP_CONFIG_LOCATION $FINAL_CONFIG_LOCATION
    if [[ -f $PIDFILE ]]; then
        haproxy -D -f $FINAL_CONFIG_LOCATION -p $PIDFILE -sf $(cat $PIDFILE)
    else
        haproxy -D -f $FINAL_CONFIG_LOCATION -p $PIDFILE
    fi
}

shutdown() {
    if [[ -f $PIDFILE ]]; then
        kill -SIGTERM $(cat $PIDFILE)
    fi
    exit
}
trap shutdown SIGKILL SIGTERM SIGINT EXIT

while inotifywait -t 60 -q -e close_write,moved_to,create /etc/hrmconfig || true; do
    if [[ -s $CONFIG_LOCATION ]] && [[ -s $CONFIG_CHECKSUM_LOCATION ]]; then
        NEW_CHECKSUM=$(sha256sum "$CONFIG_LOCATION" | awk '{print $1}')
        DESIRED_CHECKSUM=$(cat $CONFIG_CHECKSUM_LOCATION)
        if [[ "$NEW_CHECKSUM" != "$DESIRED_CHECKSUM" ]]; then
            echo "$(date) -- checksum was not desired checksum: $NEW_CHECKSUM instead of $DESIRED_CHECKSUM"
            continue
        fi

        if [[ "$NEW_CHECKSUM" != "$CHECKSUM" ]]; then
            echo "$(date) -- checksum changed, was $CHECKSUM, now $NEW_CHECKSUM"

            setup_logging_in_config $CONFIG_LOCATION "${CONFIG_LOCATION}.tmp"
            haproxy -c -f "${CONFIG_LOCATION}.tmp"
            OUT=$?
            if [[ "$OUT" -eq 1 ]]; then
                echo "$(date) -- new haproxy config did not validate"
                continue
            fi

            CHECKSUM="$NEW_CHECKSUM"
            cp $CONFIG_LOCATION $TMP_CONFIG_LOCATION
            start_haproxy
        fi
    fi
done
